@@ -3,7 +3,7 @@
 # Authors:
 #  - Arjan Verwer
 #  - Peter Odding <peter.odding@paylogic.com>
-# Last Change: December 16, 2018
+# Last Change: August 4, 2020
 # URL: https://py2deb.readthedocs.io
 
 """
@@ -30,7 +30,8 @@ from property_manager import PropertyManager, cached_property, lazy_property, mu
 from deb_pkg_tools.cache import get_default_cache
 from deb_pkg_tools.checks import check_duplicate_files
 from deb_pkg_tools.utils import find_debian_architecture
-from humanfriendly import coerce_boolean, compact
+from humanfriendly import coerce_boolean
+from humanfriendly.text import compact
 from pip_accel import PipAccelerator
 from pip_accel.config import Config as PipAccelConfig
 from six.moves import configparser
@@ -186,7 +187,7 @@ class PackageConverter(PropertyManager):
         """
         The name prefix for converted packages (a string).
 
-        The default value of :ref:`name_prefix` depends on
+        The default value of :attr:`name_prefix` depends on
         the Python interpreter that's used to run py2deb:
 
         - On PyPy_ the default name prefix is ``pypy``.
@@ -215,7 +216,7 @@ class PackageConverter(PropertyManager):
     @mutable_property
     def prerelease_workaround(self):
         """
-        Whether to enable the pre-release workaround in :func:`normalize_package_version()` (a boolean).
+        Whether to enable the pre-release workaround in :func:`.normalize_package_version()` (a boolean).
 
         By setting this to :data:`False` converted version numbers will match
         those generated by py2deb 0.25 and earlier. Release 1.0 introduced the
