@@ -1,7 +1,7 @@
 # Automated tests for the `py2deb' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.com>
-# Last Change: December 16, 2018
+# Last Change: August 5, 2020
 # URL: https://py2deb.readthedocs.io
 
 """
@@ -33,7 +33,7 @@ from deb_pkg_tools.control import load_control_file, patch_control_file
 from deb_pkg_tools.package import inspect_package, parse_filename
 from executor import execute
 from humanfriendly.text import dedent
-from humanfriendly.testing import TestCase, run_cli
+from humanfriendly.testing import TestCase, run_cli, touch
 
 # Modules included in our package.
 from py2deb.cli import main
@@ -55,7 +55,6 @@ from py2deb.hooks import (
     initialize_namespaces,
     post_installation_hook,
     pre_removal_hook,
-    touch,
 )
 
 # Initialize a logger.
@@ -70,6 +69,9 @@ TEST_NAMESPACES = [('foo',),
                    ('foo', 'bar'),
                    ('foo', 'bar', 'baz')]
 
+# Supported namespace package styles.
+NAMESPACE_STYLES = ['setuptools', 'pkgutil', 'none']
+
 
 def setUpModule():
     """
@@ -161,6 +163,11 @@ class PackageConverterTestCase(TestCase):
         assert normalize_package_version('1.0b2') == '1.0~b2'
         assert normalize_package_version('1.0c2') == '1.0~rc2'
         assert normalize_package_version('1.0rc2') == '1.0~rc2'
+        # Do not modify local version labels
+        assert normalize_package_version('1.0+a2') == '1.0+a2'
+        assert normalize_package_version('1.0+b2') == '1.0+b2'
+        assert normalize_package_version('1.0+c2') == '1.0+c2'
+        assert normalize_package_version('1.0+65c43') == '1.0+65c43'
         # New versus old behavior (the option to control backwards compatibility was added in release 2.1).
         assert normalize_package_version('1.0a2', prerelease_workaround=True) == '1.0~a2'
         assert normalize_package_version('1.0a2', prerelease_workaround=False) == '1.0a2'
@@ -348,7 +355,13 @@ class PackageConverterTestCase(TestCase):
             expression = '%s (= 3.6.0)' % fix_name_prefix('python-raven-flask')
             assert expression in relationships
             # Check that a package with the extra in the filename was generated.
-            assert find_package_archive(archives, fix_name_prefix('python-raven-flask'))
+            archive = find_package_archive(archives, fix_name_prefix('python-raven-flask'))
+            assert archive
+            # Use deb-pkg-tools to inspect the package metadata.
+            metadata, contents = inspect_package(archive)
+            logger.debug("Metadata of generated package: %s", dict(metadata))
+            # Check that a "Provides" field was added.
+            assert metadata['Provides'].matches(fix_name_prefix('python-raven'))
 
     def test_conversion_of_environment_markers(self):
         """
@@ -357,13 +370,17 @@ class PackageConverterTestCase(TestCase):
         Converts ``weasyprint==0.42`` and sanity checks that the ``cairosvg``
         dependency is present.
         """
-        if sys.version_info[:2] == (2, 6):
-            self.skipTest("WeasyPrint 0.42 is not Python 2.6 compatible")
         with TemporaryDirectory() as directory:
+            # Find our constraints file.
+            module_directory = os.path.dirname(os.path.abspath(__file__))
+            project_directory = os.path.dirname(module_directory)
+            constraints_file = os.path.join(project_directory, 'constraints.txt')
             # Run the conversion command.
             converter = self.create_isolated_converter()
             converter.set_repository(directory)
-            archives, relationships = converter.convert(['weasyprint==0.42'])
+            # Constrain tinycss2 to avoid Python 2 incompatibilities:
+            # https://travis-ci.org/github/paylogic/py2deb/jobs/713388666
+            archives, relationships = converter.convert(['--constraint=%s' % constraints_file, 'weasyprint==0.42'])
             # Check that the dependency is present.
             pathname = find_package_archive(archives, fix_name_prefix('python-weasyprint'))
             metadata, contents = inspect_package(pathname)
@@ -749,49 +766,79 @@ class PackageConverterTestCase(TestCase):
         This tests the :func:`~py2deb.hooks.initialize_namespaces()` and
         :func:`~py2deb.hooks.cleanup_namespaces()` functions.
         """
+        for namespace_style in NAMESPACE_STYLES:
+            with TemporaryDirectory() as directory:
+                package_name = 'namespace-package-test'
+                initialize_namespaces(package_name, directory, TEST_NAMESPACES, namespace_style)
+                self.check_test_namespaces(directory)
+                # Increase the reference count of the top level name space.
+                initialize_namespaces(package_name, directory, set([('foo',)]), namespace_style)
+                self.check_test_namespaces(directory)
+                # Clean up the nested name spaces.
+                cleanup_namespaces(package_name, directory, TEST_NAMESPACES)
+                # Make sure top level name space is still intact.
+                assert os.path.isdir(os.path.join(directory, 'foo'))
+                assert os.path.isfile(os.path.join(directory, 'foo', '__init__.py'))
+                # Make sure the nested name spaces were cleaned up.
+                assert not os.path.isdir(os.path.join(directory, 'foo', 'bar'))
+                assert not os.path.isfile(os.path.join(directory, 'foo', 'bar', '__init__.py'))
+                assert not os.path.isdir(os.path.join(directory, 'foo', 'bar', 'baz'))
+                assert not os.path.isfile(os.path.join(directory, 'foo', 'bar', 'baz', '__init__.py'))
+                # Clean up the top level name space as well.
+                cleanup_namespaces(package_name, directory, TEST_NAMESPACES)
+                assert not os.path.isdir(os.path.join(directory, 'foo'))
+
+    def test_pkgutil_namespaces(self):
+        """
+        Test compatibility with :mod:`pkgutil` style namespace packages.
+
+        This test fails on py2deb <= 4.0 because the two packages involved
+        both define the same pkgutil-style namespace package and this
+        causes a file conflict that's detected by py2deb, in the form of
+        a :exc:`~deb_pkg_tools.checks.DuplicateFilesFound` exception::
+
+            deb_pkg_tools.checks.DuplicateFilesFound: Found 1 duplicate file in 2 package archives!
+            -------------------------------------------------------------------------------
+            Found 1 conflict between 2 packages:
+              1. /tmp/tmpgqz6ettd/python3-backports-functools-lru-cache_1.6.1_all.deb
+              2. /tmp/tmpgqz6ettd/python3-configparser_3.7.4_all.deb
+            These packages contain 1 conflict:
+              1. /usr/lib/python3.6/dist-packages/backports/__init__.py
+            -------------------------------------------------------------------------------
+        """
         with TemporaryDirectory() as directory:
-            package_name = 'namespace-package-test'
-            initialize_namespaces(package_name, directory, TEST_NAMESPACES)
-            self.check_test_namespaces(directory)
-            # Increase the reference count of the top level name space.
-            initialize_namespaces(package_name, directory, set([('foo',)]))
-            self.check_test_namespaces(directory)
-            # Clean up the nested name spaces.
-            cleanup_namespaces(package_name, directory, TEST_NAMESPACES)
-            # Make sure top level name space is still intact.
-            assert os.path.isdir(os.path.join(directory, 'foo'))
-            assert os.path.isfile(os.path.join(directory, 'foo', '__init__.py'))
-            # Make sure the nested name spaces were cleaned up.
-            assert not os.path.isdir(os.path.join(directory, 'foo', 'bar'))
-            assert not os.path.isfile(os.path.join(directory, 'foo', 'bar', '__init__.py'))
-            assert not os.path.isdir(os.path.join(directory, 'foo', 'bar', 'baz'))
-            assert not os.path.isfile(os.path.join(directory, 'foo', 'bar', 'baz', '__init__.py'))
-            # Clean up the top level name space as well.
-            cleanup_namespaces(package_name, directory, TEST_NAMESPACES)
-            assert not os.path.isdir(os.path.join(directory, 'foo'))
+            converter = self.create_isolated_converter()
+            converter.set_repository(directory)
+            converter.convert([
+                'configparser==3.7.4',
+                'backports.functools-lru-cache==1.6.1',
+            ])
 
     def test_post_install_hook(self):
         """Test the :func:`~py2deb.hooks.post_installation_hook()` function."""
-        with TemporaryDirectory() as directory:
-            self.run_post_install_hook(directory)
-            self.check_test_namespaces(directory)
+        for namespace_style in NAMESPACE_STYLES:
+            with TemporaryDirectory() as directory:
+                self.run_post_install_hook(directory, namespace_style)
+                self.check_test_namespaces(directory)
 
     def test_pre_removal_hook(self):
         """Test the :func:`~py2deb.hooks.pre_removal_hook()` function."""
-        with TemporaryDirectory() as directory:
-            self.run_post_install_hook(directory)
-            pre_removal_hook(package_name='prerm-test-package',
-                             alternatives=set(),
-                             modules_directory=directory,
-                             namespaces=TEST_NAMESPACES)
-            assert not os.path.isdir(os.path.join(directory, 'foo'))
+        for namespace_style in NAMESPACE_STYLES:
+            with TemporaryDirectory() as directory:
+                self.run_post_install_hook(directory, namespace_style)
+                pre_removal_hook(package_name='prerm-test-package',
+                                 alternatives=set(),
+                                 modules_directory=directory,
+                                 namespaces=TEST_NAMESPACES)
+                assert not os.path.isdir(os.path.join(directory, 'foo'))
 
-    def run_post_install_hook(self, directory):
+    def run_post_install_hook(self, directory, namespace_style):
         """Helper for :func:`test_post_install_hook()` and :func:`test_pre_removal_hook()`."""
         post_installation_hook(package_name='postinst-test-package',
                                alternatives=set(),
                                modules_directory=directory,
-                               namespaces=TEST_NAMESPACES)
+                               namespaces=TEST_NAMESPACES,
+                               namespace_style=namespace_style)
 
     def check_test_namespaces(self, directory):
         """Make sure the test name spaces are properly initialized."""
